#define F_CPU 1000000UL

#include <avr/io.h>
#include <stdlib.h>
#include <util/delay.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "iocompat.h"  


#define MAXPAT 3

enum { UP, DOWN };

ISR (TIMER1_OVF_vect)           /* Note [2] */
{
    static uint16_t pwm;        /* Note [3] */
    static uint8_t direction;

    switch (direction)          /* Note [4] */
    {
        case UP:
            if (++pwm == TIMER1_TOP)
                direction = DOWN;
            break;

        case DOWN:
            if (--pwm == 0)
                direction = UP;
            break;
    }

    OCR = pwm;                  /* Note [5] */
}



void ioinit (void)                   /* Note [6] */
{
    /* Timer 1 is 10-bit PWM (8-bit PWM on some ATtinys). */
    TCCR1A = TIMER1_PWM_INIT;
    /*
     * Start timer 1.
     *
     * NB: TCCR1A and TCCR1B could actually be the same register, so
     * take care to not clobber it.
     */
    TCCR1B |= TIMER1_CLOCKSOURCE;
    /*
     * Run any device-dependent timer 1 setup hook if present.
     */
//#if defined(TIMER1_SETUP_HOOK)
 //   TIMER1_SETUP_HOOK();
//#endif

    /* Set PWM value to 0. */
    OCR = 0;

    /* Enable OC1 as output. */
    DDROC = _BV (OC1);

    /* Enable timer 1 overflow interrupt. */
    TIMSK = _BV (TOIE1);
    sei ();
}

void iodeinit (void)
{
	cli();
    TIMSK = 0;
    TCCR1A = 0;
    TCCR1B = 0;
	OCR = 0;
    DDRC = 0;
    DDRB = 0;
	PORTC = 0x00;
	PORTB = 0x00;
}



void do_pwm(){

    ioinit();

	_delay_ms(12000);
    iodeinit();
	_delay_ms(500);
}






int main (void)
{

	int mode=16;
	int delay=200;
	int k=1;
	int pat[MAXPAT]={0b001001,0b010010, 0b100100};
	int t;
	short int maxt=0x20;
	short int ready=0;


	
    DDRD = 0b11111111; //All outputs


    while(1)
    {
		if (mode==1){
			PORTD = k;
			_delay_ms(delay); 

			k=k<<1;
			if (k>0x20) k=1;

			delay--;
			if (delay<10){
				mode=2;
				delay=200;
				k=0x20;
			}
		}
		if (mode==2){
			PORTD = k;
			_delay_ms(delay); 

			k=k>>1;
			if (!k || k>0x20) k=0x20;

			delay--;
			if (delay<10){
				mode=3;
				delay=200;
				k=0;
			}
		}
		if (mode==3){
			PORTD = pat[k];
			_delay_ms(delay); 

			k++;
			if (k>MAXPAT-1)k=0;


			delay--;
			if (delay<10){
				mode=4;
				delay=100;
			}
		}
		if (mode==4){
			k=0x40;

			while(k>0x20){
				k=random();
			}

			PORTD = k;
			_delay_ms(delay); 

			if (delay>100) delay=100;
			delay--;
			if (delay<10){
				mode=5;
				delay=100;
				maxt=0x20;
				ready=0;
				k=1;
			}
		}
		if (mode==5 || mode==7 || mode==9  || mode==11  || mode==13  ){
			if (k==maxt){
				maxt=maxt>>1;
				ready=ready | k;
				t=ready | k;
				k=1;
			}
			else {
				t=ready | k;
				k=k<<1;
			}
;
			PORTD = t;
			_delay_ms(delay); 

			if (ready>=0x3F || !maxt || maxt>0x20){
				mode++;
				delay=100;
				k=0x20;
				maxt=0x20;
				ready=0x1F;
			}
		}
		if (mode==6 || mode==8  || mode==10  || mode==12  || mode==14){
			t=ready | k;
			
			k=k<<1;
			if (k>0x20){
				maxt=maxt>>1;
				k=maxt;
				ready=ready ^ maxt;
			}


			PORTD = t;
			_delay_ms(delay); 

			if (!ready && (!k || k>0x20)){
				mode++;
				delay=100;
				k=1;
				maxt=0x20;
				ready=0;
			}
		}
		if (mode==15){
			PORTD = k;
			_delay_ms(delay);
			k++;
			if (k>0x3F){
				delay=200;
				k=1;
				mode=16;
			}
		}
		if (mode==16){
		    DDRD = 0; //All outputs
			do_pwm();
			DDRD = 0b11111111; //All outputs
			delay=200;
			k=1;
			mode=1;
		}

    }
   
    return(0);
}


